#
#
#  Copyright (c) 2023 TU Eindhoven
#
#  File author(s): Federica Eduati (f.eduati@tue.nl)
#
#  Distributed under the GPLv3 License.
#  See accompanying file LICENSE.txt or copy at
#      http://www.gnu.org/licenses/gpl-3.0.html
#
# --------------------------------------------------------
#
#' cut the raw data or the peak data from microfluidics file.
#' 
#' \code{cut.data}  .
#' 
#' 
#' 
#' @param data raw data extracted from images from the microfluidics platform extracted peak info
#' @param start.frame start frame
#' @param end.frame end frame
cut.data <- function(data, start.frame=NA, end.frame=NA){
  
  if (is.na(start.frame)){
    start.cut=1
  }else{
    start.cut=which(data[,1]>=start.frame)[1]
    }
  if (is.na(end.frame)){
    end.cut=dim(data)[1]
  }else{
    end.cut=which(data[,1]>=end.frame)[1]
  }
  
  data.cut <- data[start.cut:end.cut,]
  
  return(data.cut)
}

# --------------------------------------------------------
#
#' define different experimental conditions using smoothing function to identify groups of peaks
#' 
#' \code{detect.conditions}  .
#' 
#' 
#' 
#' @param raw.data raw data extracted from images from the microfluidics platform
#' @param spar smoothing parameter for the smooth.spline function
detect.conditions <- function(raw.data, spar){
  
  plot.raw.peak.data(raw.data)
  lowpass.spline <- smooth.spline(raw.data$Frames, raw.data$Average.Intensity, spar = spar)
  samples.smooth <- predict(lowpass.spline, raw.data$Frames)
  # lines(samples.smooth$x, samples.smooth$y, col = "red", lwd = 2)
  peaks <- findpeaks(samples.smooth$y)
  # points(samples.smooth$x[peaks[,2]], peaks[,1], pch=16, col="red")
  abline(v=samples.smooth$x[peaks[,3]], col="blue")
  abline(v=samples.smooth$x[peaks[,4]], col="forestgreen")
  
  peaks <- as.list(data.frame(t(peaks)))
  print(paste0("Number of detected conditions = ", length(peaks)))
  conditions <- lapply(peaks, function(x){
    return(raw.data[x[3]:x[4],])
  })
  
  return(conditions)
}

# --------------------------------------------------------
#
#' define different experimental conditions using smoothing function to identify groups of peaks
#' 
#' \code{detect.peaks}  .
#' 
#' 
#' 
#' @param conditions data separated by conditions as output of the detect.conditions functions
#' @param spar smoothing parameter for the smooth.spline function
detect.peaks <- function(conditions, spar){
  
  peaks <- lapply(names(conditions), function(x){
    condition <- conditions[[x]]
    lowpass.spline <- smooth.spline(condition$Frames, condition$Average.Intensity, spar = 0.3)
    samples.smooth <- predict(lowpass.spline, raw.data$Frames)
    # plot.raw.peak.data(condition, main = x)
    # lines(samples.smooth$x, samples.smooth$y, col = "red", lwd = 2)
    peaks.cond <- findpeaks(samples.smooth$y, minpeakdistance = 20, npeaks = 5)
    colnames(peaks.cond) <- c("max", "max.x", "start", "end")
    return(peaks.cond)
    # points(samples.smooth$x[peaks[,2]], peaks[,1], pch=16, col="red")
  })
  names(peaks) <- names(conditions)
  # 
  
  # run this to plot raw data divided by condition and detected peaks
  # pdf("../figures/plotConditions.pdf")
  # for (x in names(conditions)){
  #     condition <- conditions[[x]]
  #     lowpass.spline <- smooth.spline(condition$Frames, condition$Average.Intensity, spar = 0.3)
  #     samples.smooth <- predict(lowpass.spline, raw.data$Frames)
  #     plot.raw.peak.data(condition, main = x)
  #     lines(samples.smooth$x, samples.smooth$y, col = "red", lwd = 2)
  #     peaks <- findpeaks(samples.smooth$y, minpeakdistance = 20, npeaks = 5)
  #     points(samples.smooth$x[peaks[,2]], peaks[,1], pch=16, col="red")
  #     start <- condition$Frames[1]
  #     end <- condition$Frames[length(condition$Frames)]
  #     mean.peaks <- mean(peaks[,1])
  #     median.peaks <- median(peaks[,1])
  #     sd <- std(peaks[,1])
  #     lines(x = c(start, end), c(mean.peaks, mean.peaks), col="red")
  #     lines(x = c(start, end), c(median.peaks, median.peaks), col="green")
  #     # readline(prompt = "Press [enter] to continue.")
  #     abline(h=42)
  # }
  # dev.off()
  #   
  return(peaks)
}


# --------------------------------------------------------
#
#' plot the raw data and the peak data overlapped 
#' 
#' \code{plot.conditions.peaks}  .
#' 
#' 
#' 
#' @param raw.data raw data extracted from images from the microfluidics platform
#' @param conditions  data separated by conditions as output of the detect.conditions functions
#' @param peaks extracted peaks info using detect.peaks function
plot.conditions.peaks <- function(raw.data, conditions, peaks){
  
  plot.raw.peak.data(raw.data)
  
  start.end <- lapply(conditions, function(condition){
    c(condition$Frames[1], condition$Frames[length(condition$Frames)])
  })
  
  abline(v=unlist(start.end), col="blue")
  
  plot.statistics <- lapply(names(conditions), function(x){
    condition <- conditions[[x]]
    start <- condition$Frames[1]
    end <- condition$Frames[length(condition$Frames)]
    
    # compute statistics considering always the first 4 peaks per condition 
    mean.peaks <- mean(peaks[[x]][1:4,1])
    median.peaks <- median(peaks[[x]][1:4,1])
    sd <- std(peaks[[x]][1:4,1])
    sd <- std_err(peaks[[x]][1:4,1])
    
    lines(x = c(start, end), c(mean.peaks, mean.peaks), col="red", lwd=2)
    lines(x = c(start, end), c(mean.peaks+sd, mean.peaks+sd), col="red", lty=2)
    lines(x = c(start, end), c(mean.peaks-sd, mean.peaks-sd), col="red", lty=2)
  })
  
}


# --------------------------------------------------------
#
#' plot the raw data and the peak data overlapped 
#' 
#' \code{plot.raw.peak.data}  .
#' 
#' 
#' 
#' @param raw.data raw data extracted from images from the microfluidics platform
#' @param peak.data  extracted peak info
#' @param start.frame end frame
#' @param end.frame end frame
plot.raw.peak.data <- function(raw.data, peak.data=NA, start.frame=NA, end.frame=NA, main="", showPeaks=F){
  
  raw.data <- cut.data(raw.data, start.frame, end.frame)
  if (is.na(peak.data)){
    plot(raw.data$Frames, raw.data$Average.Intensity, type="l", main=main, xlab="Frames", ylab="Intensity")
  }else{
    peak.data <- cut.data(peak.data, start.frame, end.frame)
    y.max <- max(c(raw.data$Average.Intensity, peak.data$Height.of.peak, peak.data$Width.heights))
    y.min <- min(c(raw.data$Average.Intensity, peak.data$Height.of.peak, peak.data$Width.heights))
    plot(raw.data$Frames, raw.data$Average.Intensity, type="l", ylim=c(y.min, y.max), main=main, xlab="Frames", ylab="Intensity")
    if (showPeaks==T){
      points(peak.data$Peak.locations..index., peak.data$Height.of.peak, pch=16, col="red")
      points(peak.data$Peak.locations..index., peak.data$Width.heights, pch=16, col="blue")
    }
  }
  
  
  # abline(h=140)
  # abline(h=140/3*2)
  # abline(h=140/3)
  
}
